/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19 janvier 2007
   derniere modification le 11/07/2008 */

#ifndef SCESCENE_H
#define SCESCENE_H

#include <SCE/interface/SCEState.h>
#include <SCE/interface/SCECamera.h>
#include <SCE/interface/SCENode.h>
#include <SCE/interface/SCEModel.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*
  Gestionnaire de scenes.
  
  Morceau principal du moteur, gestion des objets dans la scene:
  - octree/quadtree
  - BSP
  - contribution culling
  - cameras
    - mouvements avances
    - multiples
  - changement d'etats:
    - changement des arbres de partitionnement a utiliser --> BSP, quadtree, ...
    - effet global a la scene --> HDR, bloom, glow, ...
  - LOD --> textures, modeles, terrain, ...
  - tri/ordre d'affichage:
    - textures
    - shaders
    - modeles
    - materiaux
  - screenshot --> DevIL pour l'enregistrement
  - video --> Open*L... a voir sur dvp.net (OpenCV .. ?)
  - rendu multi-passes --> HDR, bloom, post-effect, ...
  - IA de traitement des donnees (cf ordre d'affichage)
  - {occlusion culling
  - scene hierarchique ...
  - ...
*/

/* NOTE: doit etre superieur ou egal a SCE_MAX_SHADER_LIGHTS sinon c'est con */
#define SCE_MAX_LIGHTS_ON_MODEL 16

/* nombre maximums de rendus "imbriques" qu'on peut faire */
#define SCE_MAX_RENDER_INFOS 8

/* etats actives pour une scene */
typedef struct sce_sscenerenderstates SCE_SSceneRenderStates;
struct sce_sscenerenderstates
{
    int frustum_culling; /* frustum culling active ? */
    int use_skybox;      /* utilisation d'une skybox ? */
    int lighting;        /* utilisation des lumieres dans la scene */
};

/* informations ponctuelles sur une scene (son etat de rendu par ex.) */
typedef struct sce_sscenemodelrenderinfos SCE_SSceneModelRenderInfos;
struct sce_sscenemodelrenderinfos
{
    SCE_SModel *model;
    int draw;
    /* lumieres a appliquer sur le modele */
    SCE_SLight *lights[SCE_MAX_LIGHTS_ON_MODEL];
    int num_lights;
};

/* informations ponctuelles sur une scene (son etat de rendu par ex.) */
typedef struct sce_sscenerenderinfos SCE_SSceneRenderInfos;
struct sce_sscenerenderinfos
{
    SCE_STexture *rendertarget;
    SCEuint cubeface;
    SCE_SCamera *camera;
    SCE_SList *models;             /* modeles devant etre rendus */
    SCE_SSceneRenderStates states; /* etats de la scene */
};

/* scene */
typedef struct sce_sscene SCE_SScene;
struct sce_sscene
{
    SCE_SNode *rootnode;     /* noeud racine */

    SCE_SList *models;       /* liste des modeles de la scene */
    SCE_SList *lights;       /* liste des lumieres de la scene */
    int num_models;          /* nombre total de modeles */

    /* informations bruts pour un rendu */
    SCE_SSceneRenderInfos render[SCE_MAX_RENDER_INFOS];
    int ri;                  /* id des infos de rendu courant */   
};

/* initialise le gestionnaire de scenes */
int SCE_Init_Scene (void);
/* quitte le gestionnaire de scenes */
void SCE_Quit_Scene (void);

/* cree une nouvelle scene */
SCE_SScene* SCE_Scene_Create (void);
/* supprime une scene */
void SCE_Scene_Delete (SCE_SScene*);

/* genere les boundings boxes de tous les modeles de la scene */
void SCE_Scene_GenerateBoundingsBoxes (SCE_SScene*);

/* retourne le noeud racine */
SCE_SNode* SCE_Scene_GetRootNode (SCE_SScene*);

/* ajoute un noeud au noeud racine */
int SCE_Scene_AddNode (SCE_SScene*, SCE_SNode*);
/* ajoute un modele a une scene */
int SCE_Scene_AddModel (SCE_SScene*, SCE_SModel*);
/* ajoute une skybox a une scene */
int SCE_Scene_AddSkybox (SCE_SScene*, SCE_SModel*);
/* ajoute une lumiere a une scene */
int SCE_Scene_AddLight (SCE_SScene*, SCE_SLight*);

/* callback de rendu par defaut d'une skybox */
void SCE_Scene_RenderSkybox (SCE_SModel*, void*);

/* prepare la scene pour le rendu */
void SCE_Scene_PrepareRendering (SCE_SScene*);

/* push une update dans la liste */
void SCE_Scene_PushUpdate (SCE_SScene*);
/* pop une update dans la liste */
void SCE_Scene_PopUpdate (SCE_SScene*);

/* met a jour la scene et stocke les informations de rendu dans
   l'element courant de la liste des infos de rendu */
void SCE_Scene_Update (SCE_SScene*, SCE_SCamera*, SCE_STexture*, SCEuint);
/* rend une scene a partir des infos courantes de rendu */
void SCE_Scene_Render (SCE_SScene*);


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
